use std::collections::HashMap;

use config::Value;
use mysql_async;

pub fn setup_db(settings: HashMap<String, Value>) -> mysql_async::Pool {
    let username = match settings.get("username") {
        None => None,
        Some(name) => Some(name.to_string()),
    };

    let db_name = match settings.get("db_name") {
        None => None,
        Some(name) => Some(name.to_string()),
    };

    let password = match settings.get("password") {
        None => None,
        Some(pass) => Some(pass.to_string()),
    };

    let host = settings
        .get("hostname")
        .expect("Database config is expecting a hostname/ip address")
        .clone()
        .into_string()
        .unwrap();

    println!(
        "Connecting to DB Host: {} User: {:?} DB: {:?}",
        host, username, db_name
    );

    let db_options = mysql_async::OptsBuilder::default()
        .db_name(db_name)
        .ip_or_hostname(host)
        .user(username)
        .pass(password);
    mysql_async::Pool::new(db_options)
}
